#tree_details {
    padding:0;
    padding-top:6px;
    top: 29px;
}


#id_autocomplete {
    width: 165px;
}

ul.ui-autocomplete{
    background: #ffffff;
    border:solid 1px #D4D6D8;
    width: 165px;
    max-height: 400px;
    overflow: hidden;
    overflow-y: scroll;
}

.ui-menu-item a.ui-state-focus{
    background: #D4D6D8;
}

.ui-menu-item a{
    background:#ffffff;
    font-size: 10px;
    width: 135px;
}

#autocompletesearch {
    /* override .filtersearch width from webclient */
    width: 180px;
}

.autocompletesearch label{
    top: 5px;
}

input[type='text'].ui-autocomplete-loading {
    background: url('../../webgateway/img/spinner.gif') no-repeat right center;
}


div label input {
    margin-right:100px;
}
body {
    font-family:sans-serif;
}



.case_sensitive input[type=checkbox] {
    display: inline-block;
    vertical-align: middle;
}
.case_sensitive label {
    color: #555555;
    font-weight: lighter;
    display: inline-block;
    width: 25px;
    height: 16px;
    padding: 5px 0px;
    vertical-align: middle;
}
.case_sensitive input[type=checkbox][disabled] + label{
    color: hsl(215,20%,75%);
}